import json
import asyncio
import time

# Include dbridge server library package
from databridges_sio_server_lib import dBridges
from databridges_sio_server_lib.exceptions import dBError
import wikipedia


class wikiClass():
    def __init__(self):
        self.bridge = None
        self.wikiServer = None

        # Replace your dataBrdiges Application Key,Application Secret and Authentication URL below.
        self.dbApplnKey = 'appkey'
        self.dbAuthURL = 'auth_url'
        self.dbApplnSecret =  'appsecret'

#Bind to disconnected event, to get intimation about dataBridges network disconnection.
    async def disconnected(self):
        print("disconnected:")

#Bind to connected event, to get intimation about dataBridges network connected.
    async def connected(self):
        print("connected:")
        try:
            #  Register RPC Server with dBrdige network.
            await self.wikiServer.register()
        except dBError as dberror:
            #  Catch any registration error.
            print("Exception: ", dberror.source, dberror.code, dberror.message)

        try:
             #  Bind to server.registration.fail, This will be triggered When any registration issue is raised.
            async def registrationfail(payload, metadata):
                print("dbridges:rpc.server.registration.fail: ", payload, metadata)
            self.wikiServer.bind("dbridges:rpc.server.registration.fail", registrationfail)

            #  Bind to server.registration.success, This will be triggered When any registration success is raised.
            async def registrationsuccess(payload, metadata):
                print("dbridges:rpc.server.registration.success: ", payload, metadata)
            self.wikiServer.bind("dbridges:rpc.server.registration.success", registrationsuccess)

             # Bind to server.online, This will be triggered When rpc server is ready to serve.
            async def serveronline(payload, metadata):
                print("dbridges:rpc.server.online: ", payload, metadata)

            self.wikiServer.bind("dbridges:rpc.server.online", serveronline)
        except dBError as dberror:
            print("Exception: ", dberror.source, dberror.code, dberror.message)

    # Define all exposed functions.
    async def wikiServerFunctions(self):

        async def summary(inparameter, response): 
            payload = inparameter["inparam"]
            try:
                start_time = time.time()
                data = wikipedia.summary(payload)
                jsonobject = {"statuscode": 0, "exectime": (time.time() - start_time), "data": data}
                await response.end(json.dumps(jsonobject))
            except Exception as e:
                jsonobject = {"statuscode": 1,"exectime": (time.time() - start_time), "data": ""}
                await response.end(json.dumps(jsonobject))
                
        try:
            #  Bind the exposed function, so that library can register it for receiving calls.
            self.wikiServer.regfn("summary", summary)
        except dBError as dberror:
            print("Exception: ", dberror.source, dberror.code, dberror.message)

   # main function to be executed
    async def MainExecute(self):
        #  Initialize dbridge client
        self.dbridge = dBridges()
        #  Replace your appliation key below, which you have received from dBridge management portal.
        self.dbridge.appkey = self.dbApplnKey;

        #  Replace your authentication url below, which you have received from dBridge management portal.
        self.dbridge.auth_url = self.dbAuthURL;

        #  Replace your appsecret below, which you have received from dBridge management portal.
        self.dbridge.appsecret = self.dbApplnSecret;

        # Initialize RPC Server wiki.search.
        self.wikiServer = self.dbridge.rpc.init("wiki.search")
        self.wikiServer.functions = self.wikiServerFunctions

        try:
            # Bind to connected event, Once the dbridge is connected successfully, Subscription and other activities can be performed.
            self.dbridge.connectionstate.bind("connected", self.connected)
            # Bind to disconnected event, to get intimation about dbrdige network disconnection.
            self.dbridge.connectionstate.bind("disconnected", self.disconnected)
        except dBError as dberror:
            print("Exception: ", dberror.source, dberror.code, dberror.message)

        try:
            await asyncio.sleep(0.0001)
            #  Connect to dbridge. If any runtime error it will be caught in catch().
            await self.dbridge.connect()
        except dBError as dberror:
            print("Exception: ", dberror.source, dberror.code, dberror.message)


if __name__ == '__main__':
    try:
        wikiCls = wikiClass()
        asyncio.run(wikiCls.MainExecute())
    except Exception as e:
        print(e)


